/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef RTAP_HEADER_HH_
# define RTAP_HEADER_HH_

# include <wipal/phy/phy.hh>
# include <wipal/tool/microseconds.hh>

/// Radiotap %header related compounds.
namespace rtap
{

  /// Radiotap %header format.
  struct header
  {

    enum field
    {
      tsft		= 0x00000001,
      flags		= 0x00000002,
      rate		= 0x00000004,
      channel		= 0x00000008,
      fhss		= 0x00000010,
      dbm_antsignal	= 0x00000020,
      dbm_antnoise	= 0x00000040,
      lock_quality	= 0x00000080,
      tx_attenuation	= 0x00000100,
      db_tx_attenuation	= 0x00000200,
      dbm_tx_power	= 0x00000400,
      antenna		= 0x00000800,
      db_antsignal	= 0x00001000,
      db_antnoise	= 0x00002000,
      ext		= 0x10000000,
    };

    uint8_t	it_version;	///< Must be zero.
    uint8_t	it_pad;		///< Padding; undefined.
    uint16_t	it_len;		///< Header's entire length.
    uint32_t	it_present;	///< Available fields.
  } __attribute__((__packed__));

  /**
   * Throw @c std::invalid_argument when a radiotap header is not valid.
   *
   * A radiotap header is considered invalid when:
   * <ul>
   *   <li>its first header is truncated, or</li>
   *   <li>its @c it_version field is non-zero.</li>
   * </ul>
   */
  void
  check(const header* h, size_t caplen);

  /**
   * Get the address of a radiotap %header's first field (or 0 if not
   * available.)
   *
   * @warning	This function does not check the radiotap header's version.
   */
  const void*
  first_field(const header*, size_t caplen);

  /**
   * Get a pointer to the IEEE 802.11 frame (or 0 if not available.)
   *
   * @warning	This function does not check the radiotap header is valid.
   * @see	check()
   */
  const void*
  get_80211(const header*, size_t caplen);

  /**
   * Retrieve a radiotap header's TSFT field, or throw @c
   * std::invalid_argument.
   *
   * @warning	This function does not check the radiotap header is valid.
   * @see	check()
   */
  tool::microseconds
  get_tsft(const header*, size_t caplen, const void* first_field);

  /**
   * Compare IEEE 802.11 bytes from radiotap headers.
   *
   * Return whether two IEEE 802.11 frames embedded inside radiotap
   * headers are equals.  This function <b>does not compare</b> the
   * radiotap headers.
   *
   * In case there is not enough bytes to decide whether frames are
   * equal (caplen is too short), consider they are not.  This is
   * a conservative approach: when merging two traces, this will lead
   * to both frames having their entry in the merged trace.
   */
  bool
  eq_80211(const header* lhs, size_t lhs_caplen,
	   const header* rhs, size_t rhs_caplen);

  /**
   * Compare IEEE 802.11 bytes from radiotap headers, but also check
   * MAC time.
   *
   * Return whether two IEEE 802.11 frames embedded inside radiotap
   * headers are equals.  Also compare the TSFT fields of radiotap
   * headers.  When the time difference between each header's field is
   * less than @c precision, then consider the field equals.
   *
   * In case there is not enough bytes to decide whether frames are
   * equal (caplen is too short), consider they are not.  This is
   * a conservative approach: when merging two traces, this will lead
   * to both frames having their entry in the merged trace.
   */
  bool
  eq_time_and_80211(const header* lhs, size_t lhs_caplen,
		    const header* rhs, size_t rhs_caplen,
		    unsigned	precision = phy::eq_time_default_precision);

} // End of namespace rtap.

# include "rtap_header.hxx"

#endif // ! RTAP_HEADER_HH_
