/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_SYNCHRONIZER_HH_
# define WIFI_FRAME_FILTER_SYNCHRONIZER_HH_

# include <utility>

# include <gmpxx.h>

# include <wipal/tool/valued_iterator.hh>
# include <wipal/tool/iterable.hh>
# include <wipal/wifi/addr_mapping.hh>

# include "linear_regression_synchronizer_fwd.hh"

namespace tool
{

  template <class I, class B1, class B2>
  struct types< wifi::frame::filter::internals::lr_sync_iterator<I, B1, B2> >
  {
    typedef wifi::frame::filter::
      synchronized_frame<typename I::value_type::value_type, mpf_class>
      value_type;
  };

  template <class I, class B>
  struct types< wifi::frame::filter::linear_regression_synchronizer<I, B> >
  {
    typedef
      wifi::frame::filter::internals::lr_sync_iterator<I, B, bottom>
      iterator;
  };

} // End of namespace tool.

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      template <class OriginalFrameType, class ImplType>
      struct synchronized_frame: public OriginalFrameType
      {
	typedef ImplType					impl_type;
	typedef typename std::pair<impl_type, impl_type>	coefs_type;

	synchronized_frame(const OriginalFrameType&, const coefs_type&);

	coefs_type	coefs;
      };

      namespace internals
      {

	template <class I, class B, class Bottom = tool::bottom>
	struct lr_sync_iterator: wp_inherit(public tool::valued_iterator,
					    lr_sync_iterator<I, B, Bottom>)
	{
	  /// @name Static hierarchies related types.
	  //@{
	  typedef wp_get_exact(Bottom, lr_sync_iterator<I, B, Bottom>)
								exact_type;
	  typedef tool::valued_iterator<exact_type>		super_type;
	  typedef wp_type(value_type, exact_type)		value_type;
	  //@}
	  /// Iterable class that produces instances of this class.
	  typedef linear_regression_synchronizer<I, B>		iterable_type;

	  /// tool::iterable compatible constructor.
	  lr_sync_iterator(const iterable_type& iterable, bool end);

	  /// @name Abstract method implementations.
	  //@{
 	  bool			equal(const exact_type&) const;
	  void			increment();
	  //@}

	private:
	  const iterable_type*	iterable_;
	  I			next_;
	  unsigned		next_wpos_;
	};

      } // End of namespace wifi::frame::filter::internals.

      template <class InputIterator, class Bottom = tool::bottom>
      struct linear_regression_synchronizer:
	wp_inherit(public tool::iterable,
		   linear_regression_synchronizer<InputIterator, Bottom>)
      {
	linear_regression_synchronizer(const InputIterator& first,
				       const InputIterator& last);

      private:
	InputIterator	first_;
	InputIterator	last_;

	friend class internals::
	  lr_sync_iterator<InputIterator, Bottom, tool::bottom>;
      };

      /**
       * Build a linear_regression_synchronizer filter from iterators
       * and pass it as argument to a functor.
       *
       * If @c filter_prism is true, filter elements with @c
       * non_noisy_prism. @c *first and @c *last should be convertible
       * to @c pcapxx::frame_descriptor.
       *
       * @c phy_end indicates the PHY headers' endianness.
       */
      template <class U, class HT, template <class, class, class> class Int,
		class I1, class I2, class F, class BL>
      void
      provide_lr_synchronizer(const I1& first1, const I1&	last1,
			      const I2& first2, const I2&	last2,
			      addr_mapping&			mapping,
			      F&				func,
			      bool				filter_prism,
			      tool::endian::endianness		phy_end,
			      const BL&				blist);

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

# include "linear_regression_synchronizer.hxx"

#endif // ! WIFI_FRAME_FILTER_SYNCHRONIZER_HH_
