/*
 * WiPal - A library and a set of tools to manipulate wireless traces.
 * Copyright (C) 2007  Universite Pierre et Marie Curie - Paris 6
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA
 *
 * Author: Thomas Claveirole <thomas.claveirole@lip6.fr>
 */
#ifndef WIFI_FRAME_FILTER_UNIQUELY_IDENTIFIABLE_HXX_
# define WIFI_FRAME_FILTER_UNIQUELY_IDENTIFIABLE_HXX_

# include <wipal/wifi/frame/dissector/dissector.hh>
# include <wipal/wifi/frame/unique_id/unique_id.hh>
# include <wipal/wifi/frame/filter/non_noisy_prism.hh>

namespace wifi
{
  namespace frame
  {
    namespace filter
    {

      namespace internals
      {

	template <class U, class I, class H, class B1, class B2>
	uniquely_identifiable_iterator<U, I, H, B1, B2>::
	uniquely_identifiable_iterator(const iterable_type&	uif,
				       bool			end):
	  super_type (),
	  iterable_ (&uif),
	  next_ (end ? uif.end_ : uif.begin_)
	{
	  if (not end)
	    increment();
	}

	template <class U, class I, class H, class B1, class B2>
	bool
	uniquely_identifiable_iterator<U, I, H, B1, B2>::
	equal(const uniquely_identifiable_iterator& rhs) const
	{
	  if (not rhs.value())
	    return not this->value();
	  return next_ == rhs.next_;
	}

	template <class U, class I, class H, class B1, class B2>
	void
	uniquely_identifiable_iterator<U, I, H, B1, B2>::increment()
	{
	  using tool::endian::need_swap;

	  boost::optional<U>	id;

	  while (next_ != iterable_->end_ and not id)
	    {
	      const pcapxx::pkthdr*	p = next_->pcap_header().get();
	      const H*			h =
		reinterpret_cast<const H*> (next_->bytes().get());

	      mactime_.tick(h, p->caplen,
			    need_swap(iterable_->phy_end_, p->swapped));
	      id = make_unique_id<U, H>(*next_, *iterable_->mapping_);
	      ++next_;
	    }
	  if (id)
	    this->value() = value_type (mactime_.microseconds(), id.get());
	  else
	    this->value() = boost::none_t ();
	}

      } // End of namespace wifi::frame::filter::internals.

      template <class U, class I, class H, class B>
      uniquely_identifiable<U, I, H, B>::
      uniquely_identifiable(const I&			begin,
			    const I&			end,
			    addr_mapping&		mapping,
			    tool::endian::endianness	phy_end):
	begin_ (begin),
	end_ (end),
	mapping_ (&mapping),
	phy_end_ (phy_end)
      {
      }

      template <class U, class HT, class I, class F>
      void
      for_each_uniquely_identifiable(const I&			first,
				     const I&			last,
				     addr_mapping&		mapping,
				     F&				func,
				     bool			filter_prism,
				     tool::endian::endianness	phy_end)
      {
	if (filter_prism)
	  {
	    typedef non_noisy_prism<I>				filter_type;
	    typedef typename filter_type::iterator		iterator;
	    typedef uniquely_identifiable<U, iterator, HT>	unique_type;

	    filter_type	f (first, last);
	    unique_type	u (f.begin(), f.end(), mapping, phy_end);

	    u.for_each(func);
	  }
	else
	  {
	    typedef uniquely_identifiable<U, I, HT>		unique_type;

	    unique_type	u (first, last, mapping, phy_end);

	    u.for_each(func);
	  }
      }

      // FIXME: Avoid code duplication with the above. :-( :-(
      template <class U, class HT, class I, class F>
      void
      for_each_uniquely_identifiable(const I&			first,
				     const I&			last,
				     addr_mapping&		mapping,
				     const F&			func,
				     bool			filter_prism,
				     tool::endian::endianness	phy_end)
      {
	if (filter_prism)
	  {
	    typedef non_noisy_prism<I>				filter_type;
	    typedef typename filter_type::iterator		iterator;
	    typedef uniquely_identifiable<U, iterator, HT>	unique_type;

	    filter_type	f (first, last);
	    unique_type	u (f.begin(), f.end(), mapping, phy_end);

	    u.for_each(func);
	  }
	else
	  {
	    typedef uniquely_identifiable<U, I, HT>		unique_type;

	    unique_type	u (first, last, mapping, phy_end);

	    u.for_each(func);
	  }
      }

    } // End of namespace wifi::frame::filter.

  } // End of namespace wifi::frame.

} // End of namespace wifi.

#endif // ! WIFI_FRAME_FILTER_UNIQUELY_IDENTIFIABLE_HXX_
